/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml.sax;

import org.jitterbit.system.callback.IAccepter;

/**
 * Utility class for extracting the value of an element when parsing XML using a
 * {@link DefaultKongaSaxParser}.
 * 
 * @author Torgil
 * 
 * @since 1.2
 * 
 */
public class SingleElementValueHandler {

    private final IEndElementHandler    m_handler;
    private final IAccepter<String>     m_callback;
    
    private SingleElementValueHandler(DefaultKongaSaxParser parser,
                                     String elementName,
                                     IAccepter<String> valueCallback) {
        m_callback = valueCallback;
        m_handler = new Handler();
        parser.addEndElementHandlerFactory(elementName, new IEndElementHandlerFactory() {

            public IEndElementHandler getHandler() {
                return m_handler;
            }
        });
    }
    
    /**
     * Registers an end element handler for the specified element name.
     * 
     * @param parser
     *            the parser that is parsing the XML.
     * @param elementName
     *            the name of the element, from which to extract the value.
     * @param valueCallback
     *            an IAccepter that is passed the value of the element, once it
     *            has been extracted.
     */
    public static void register(DefaultKongaSaxParser parser,
                                String elementName,
                                IAccepter<String> valueCallback) {
        if (parser == null) {
            throw new IllegalArgumentException("parser cannot be null.");
        }
        if (elementName == null) {
            throw new IllegalArgumentException("elementName cannot be null.");
        }
        if (valueCallback == null) {
            throw new IllegalArgumentException("valueCallback cannot be null.");
        }
        new SingleElementValueHandler(parser, elementName, valueCallback);
    }

    
    private class Handler implements IEndElementHandler {

        public void handleEnd(IKongaSaxParser parser) throws KongaSaxParserException {
            String value = parser.getCurrentElementValue();
            m_callback.accept(value);
        }
    }
}
